diff --git a/build/win32/libx264.vcproj b/build/win32/libx264.vcproj
index 8497a35..a0265fc 100644
diff --git a/build/win32/x264.sln b/build/win32/x264.sln
index 8fb518d..b4d5c5a 100644
diff --git a/build/win32/x264.vcproj b/build/win32/x264.vcproj
index c567265..48947c2 100644
diff --git a/common/common.c b/common/common.c
index 9260c64..835f20e 100644
--- a/common/common.c
+++ b/common/common.c
@@ -72,6 +72,7 @@ void    x264_param_default( x264_param_t *param )
     param->i_bframe_adaptive = X264_B_ADAPT_FAST;
     param->i_bframe_bias = 0;
     param->b_bframe_pyramid = 0;
+    param->b_strictdpb = 0;
 
     param->b_deblocking_filter = 1;
     param->i_deblocking_filter_alphac0 = 0;
@@ -350,6 +351,8 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
         p->i_bframe_bias = atoi(value);
     OPT("b-pyramid")
         p->b_bframe_pyramid = atobool(value);
+    OPT("strictdpb")
+        p->b_strictdpb = atobool(value);
     OPT("nf")
         p->b_deblocking_filter = !atobool(value);
     OPT2("filter", "deblock")
@@ -860,8 +863,8 @@ char *x264_param2string( x264_param_t *p, int b_res )
     s += sprintf( s, " bframes=%d", p->i_bframe );
     if( p->i_bframe )
     {
-        s += sprintf( s, " b_pyramid=%d b_adapt=%d b_bias=%d direct=%d wpredb=%d",
-                      p->b_bframe_pyramid, p->i_bframe_adaptive, p->i_bframe_bias,
+        s += sprintf( s, " b_pyramid=%d b_strictdpb=%d b_adapt=%d b_bias=%d direct=%d wpredb=%d",
+                      p->b_bframe_pyramid, p->b_strictdpb, p->i_bframe_adaptive, p->i_bframe_bias,
                       p->analyse.i_direct_mv_pred, p->analyse.b_weighted_bipred );
     }
 
diff --git a/common/common.h b/common/common.h
index 1e46ae8..a7b6ee5 100644
--- a/common/common.h
+++ b/common/common.h
@@ -203,6 +203,11 @@ typedef struct
         int idc;
         int arg;
     } ref_pic_list_order[2][16];
+    struct // for future expansion
+    {
+        int i_difference_of_pic_nums;
+    } mmco[16];
+    int i_mmco_idx;
 
     int i_cabac_init_idc;
 
@@ -363,7 +368,12 @@ struct x264_t
     x264_frame_t    *fref1[16+3];     /* ref list 1 */
     int             b_ref_reorder[2];
 
-
+    /* mmco command list */
+    int             i_mmco_idx;
+    struct
+    {
+        int         i_difference_of_pic_nums;
+    } mmco[16];
 
     /* Current MB DCT coeffs */
     struct
diff --git a/encoder/encoder.c b/encoder/encoder.c
index fdcc957..2d1ccf3 100644
--- a/encoder/encoder.c
+++ b/encoder/encoder.c
@@ -125,6 +125,15 @@ static void x264_slice_header_init( x264_t *h, x264_slice_header_t *sh,
 
     /* If the ref list isn't in the default order, construct reordering header */
     /* List1 reordering isn't needed yet */
+    sh->i_mmco_idx = h->i_mmco_idx;
+    if( h->i_mmco_idx )
+    {
+        int j = 0;
+        for( ; j < h->i_mmco_idx; j++ )
+        {
+            sh->mmco[j].i_difference_of_pic_nums = h->mmco[j].i_difference_of_pic_nums;
+        }
+    }
     if( sh->b_ref_pic_list_reordering_l0 )
     {
         int pred_frame_num = i_frame;
@@ -238,7 +247,6 @@ static void x264_slice_header_write( bs_t *s, x264_slice_header_t *sh, int i_nal
             {
                 bs_write_ue( s, sh->ref_pic_list_order[0][i].idc );
                 bs_write_ue( s, sh->ref_pic_list_order[0][i].arg );
-
             }
             bs_write_ue( s, 3 );
         }
@@ -251,7 +259,7 @@ static void x264_slice_header_write( bs_t *s, x264_slice_header_t *sh, int i_nal
             for( i = 0; i < sh->i_num_ref_idx_l1_active; i++ )
             {
                 bs_write_ue( s, sh->ref_pic_list_order[1][i].idc );
-                bs_write_ue( s, sh->ref_pic_list_order[1][i].arg );
+                bs_write_ue( s, sh->ref_pic_list_order[1][i].idc );
             }
             bs_write_ue( s, 3 );
         }
@@ -272,7 +280,17 @@ static void x264_slice_header_write( bs_t *s, x264_slice_header_t *sh, int i_nal
         }
         else
         {
-            bs_write1( s, 0 );  /* adaptive_ref_pic_marking_mode_flag */
+            bs_write1( s, sh->i_mmco_idx > 0 ); /* adaptive_ref_pic_marking_mode_flag */
+            if( sh->i_mmco_idx > 0 )
+            {
+                int i = 0;
+                for( ; i < sh->i_mmco_idx; i++ )
+                {
+                    bs_write_ue( s, 1 ); /* mark short term ref as unused */
+                    bs_write_ue( s, sh->mmco[i].i_difference_of_pic_nums - 1 );
+                }
+                bs_write_ue( s, 0 );
+            }
         }
     }
 
@@ -981,11 +999,16 @@ static inline void x264_reference_build_list( x264_t *h, int i_poc )
     if( h->sh.i_type == SLICE_TYPE_P )
     {
         for( i = 0; i < h->i_ref0 - 1; i++ )
+        {
             if( h->fref0[i]->i_frame_num < h->fref0[i+1]->i_frame_num )
             {
                 h->b_ref_reorder[0] = 1;
                 break;
             }
+        }
+        if( h->param.b_bframe_pyramid )
+            h->b_ref_reorder[0] = 1;
+        
     }
 
     h->i_ref1 = X264_MIN( h->i_ref1, h->frames.i_max_ref1 );
@@ -1107,6 +1130,39 @@ static inline void x264_reference_update( x264_t *h )
     h->fdec = x264_frame_pop_unused( h );
 }
 
+static inline void x264_reference_partial_reset( x264_t *h )
+{
+    int refs = 0;
+    int lastnonbidx = -1;
+    while( h->frames.reference[refs] )
+    {
+        if( h->frames.reference[refs]->i_type == X264_TYPE_I
+         || h->frames.reference[refs]->i_type == X264_TYPE_IDR
+         || h->frames.reference[refs]->i_type == X264_TYPE_P )
+            lastnonbidx = refs;
+        refs++;
+    }
+    assert( lastnonbidx > -1 );
+    // This makes room in the dpb for the delayed display time of the later b-frame's
+    // Within a mini-gop
+    while( h->param.b_bframe_pyramid && ( ( refs + 2 ) > X264_MAX( h->frames.i_max_dpb, 4 ) ) 
+                                     && h->sh.i_type ==  SLICE_TYPE_B )
+    {
+        int diff = h->i_frame_num - h->frames.reference[0]->i_frame_num;
+        h->mmco[ h->i_mmco_idx++ ].i_difference_of_pic_nums = diff;
+        x264_frame_push_unused( h, x264_frame_shift( h->frames.reference ) );
+        lastnonbidx--;
+        refs--;
+    }
+    while( refs > lastnonbidx+1 )
+    {
+        int diff = h->i_frame_num - h->frames.reference[refs-1]->i_frame_num;
+        h->mmco[ h->i_mmco_idx++ ].i_difference_of_pic_nums = diff;
+        x264_frame_push_unused( h, x264_frame_pop( h->frames.reference ) );
+        refs--;
+    }
+}
+
 static inline void x264_reference_reset( x264_t *h )
 {
     while( h->frames.reference[0] )
@@ -1435,6 +1491,7 @@ int     x264_encoder_encode( x264_t *h,
             x264_frame_expand_border_mod16( h, fenc );
 
         fenc->i_frame = h->frames.i_input++;
+        h->i_mmco_idx = 0;
 
         x264_frame_push( h->frames.next, fenc );
 
@@ -1519,12 +1576,15 @@ int     x264_encoder_encode( x264_t *h,
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* Not completely true but for now it is (as all I/P are kept as ref)*/
         h->sh.i_type = SLICE_TYPE_P;
+        x264_reference_partial_reset( h );
     }
     else if( h->fenc->i_type == X264_TYPE_BREF )
     {
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* maybe add MMCO to forget it? -> low */
         h->sh.i_type = SLICE_TYPE_B;
+        if( h->param.b_strictdpb )
+            x264_reference_partial_reset( h );
     }
     else    /* B frame */
     {
diff --git a/x264.c b/x264.c
index 6e04edd..4d6352f 100644
--- a/x264.c
+++ b/x264.c
@@ -191,6 +191,7 @@ static void Help( x264_param_t *defaults, int b_longhelp )
         "                                  - 2: Optimal (slow with high --bframes)\n", defaults->i_bframe_adaptive );
     H1( "      --b-bias <integer>      Influences how often B-frames are used [%d]\n", defaults->i_bframe_bias );
     H0( "      --b-pyramid             Keep some B-frames as references\n" );
+    H0( "      --strictdpb             Stricter usage of dpb when b-pyramid is active (ref 4+)\n" );
     H0( "      --no-cabac              Disable CABAC\n" );
     H0( "  -r, --ref <integer>         Number of reference frames [%d]\n", defaults->i_frame_reference );
     H1( "      --no-deblock            Disable loop filter\n" );
@@ -384,6 +385,7 @@ static struct option long_options[] =
     { "no-b-adapt",        no_argument, NULL, 0 },
     { "b-bias",      required_argument, NULL, 0 },
     { "b-pyramid",         no_argument, NULL, 0 },
+    { "strictdpb",         no_argument, NULL, 0 },
     { "min-keyint",  required_argument, NULL, 'i' },
     { "keyint",      required_argument, NULL, 'I' },
     { "scenecut",    required_argument, NULL, 0 },
diff --git a/x264.h b/x264.h
index e61040e..683694f 100644
--- a/x264.h
+++ b/x264.h
@@ -191,6 +191,7 @@ typedef struct x264_param_t
     int         i_bframe_adaptive;
     int         i_bframe_bias;
     int         b_bframe_pyramid;   /* Keep some B-frames as references */
+    int         b_strictdpb;        /* Reserve a spot in dpb for delayed display time b frames */
 
     int         b_deblocking_filter;
     int         i_deblocking_filter_alphac0;    /* [-6, 6] -6 light filter, 6 strong */